--Handles all code for getting and freeing tables
--Basically use a table cache so not constantly creating new tables to be garbage collected
local Recount2 = Recount2

local Tables={}
local MAX_STORED_TABLES = 20
function Recount2:FreeTable(t)
	if type(t)~="table" then
		return
	end

	--Don't store tables if currently at max tables
	if #Tables>=MAX_STORED_TABLES then
		return
	end

	--Don't free tables marked to not be free
	if t.DontFree then
		return
	end

	for k in pairs(t) do
		t[k]=nil
	end

	for _,v in pairs(Tables) do
		if v==t then
			return
		end
	end

	tinsert(Tables,t)
end

function Recount2:FreeTableRecurse(t)
	--Don't store tables if currently at max tables
	if #Tables>=MAX_STORED_TABLES then
		return
	end

	--Check the table first before recursing
	for _,v in pairs(Tables) do
		if v==t then
			return
		end
	end

	for k in pairs(t) do
		--Make sure we are allowed to free this table
		if type(t[k])=="table" then
			if t[k].DontFree~=true then
				Recount2:FreeTableRecurse(t[k])
				t[k]=nil
			end
		else
			t[k]=nil		
		end
	end

	tinsert(Tables,t)
end

function Recount2:FreeTableRecurseLimit(t,depth)
	--Check the table first before recursing
	if depth<0 then
		return
	end

	--Don't store tables if currently at max tables
	if #Tables>=MAX_STORED_TABLES then
		return
	end

	for k in pairs(t) do
		--Make sure we are allowed to free this table
		if type(t[k])=="table" then
			if t[k].DontFree~=true then
				Recount2:FreeTableRecurseLimit(t[k], depth-1)
				t[k]=nil
			end
		else
			t[k]=nil		
		end
	end

	tinsert(Tables,t)
end

if Recount2.Debug then
	function Recount2:GetTable()
		local t
		if #Tables>0 then
			t=tremove(Tables)
			
			if #t>0 then
				Recount2:Print("WARNING! For some reason there is "..#t.." entries left. There is probably a table in use that shouldn't have been freed")
			end
			for k, v in pairs(t) do
				Recount2:Print("WARNING! Table has left in it ", k, v)
			end
			return t
		else
			return {}
		end
	end
else
	function Recount2:GetTable()
		if #Tables>0 then
			return tremove(Tables)
		else
			return {}
		end
	end
end

function Recount2:HowManyTables(str)
	if str==nil then
		str=""
	else
		str=str.." "
	end
	Recount2:Print(str.."Free Tables: "..#Tables)
end

function Recount2:ResetTableCache()
	Tables=Recount2:GetTable()
end